#!/bin/bash
#   Hitachi Fibre Channel Adapter Driver Installer for Linux IA32/IA64
#   Copyright (C) 2009, 2010, Hitachi, Ltd.
#
#   $Id: hfcinst.sh,v 1.19.2.8.2.1.6.3 2010/05/22 02:13:24 eteshima Exp $
#
if [ "$1" != "install" -a "$1" != "uninstall" ]; then
        echo "Usage : hfcinst.sh install/uninstall [Kernel-Version]"  
        echo "example(install  ) : hfcinst.sh install" 
        echo "       (uninstall) : hfcinst.sh uninstall" 
        exit 1
fi
DRVDIR="drivers"
DRVMODU="hfcldd"
CNFMODU="hfcldd_conf"
MPMODU="hfcldd_mp"
HOSTTYPE=$(uname -m)
TOOLSDIR="/opt/hitachi/drivers/hba"
INSTLOG="/tmp/hfcldd_install.log"
KERNELMIN=`uname -r | sed -e 's,[^\.]*\.,,' -e 's,\..*,,'`
if [ "$HOSTTYPE" = "ia64" ]; then
	BOOTDIR="boot/efi/efi/redhat"
else
	BOOTDIR="boot"
fi
echo "      Boot directory      : $BOOTDIR" > $INSTLOG
if [ -n "$2" ]; then
	KERNVER=$2
else
	KERNVER=`uname -r`
fi
if [ $KERNELMIN = "6" ]; then
	MODCONF="modprobe.conf"
	if [ ! -f "/etc/$MODCONF" ]; then
		echo " ---- No configuration file was found (/etc/$MODCONF)"  >> $INSTLOG
		exit 1
	fi
	DRVNAME="hfcldd.ko"
	CNFNAME="hfcldd_conf.ko"
    RLOGLINK="hraslog_link.ko"
else
	MODCONF="modules.conf"
	if [ ! -f "/etc/$MODCONF" ]; then
		echo " ---- No configuration file was found (/etc/$MODCONF)"  >> $INSTLOG
		exit 1
	fi
	DRVNAME="hfcldd.o"
	CNFNAME="hfcldd_conf.o"
	RLOGLINK="hraslog_link.o"
fi
echo "      Configulation file  : $MODCONF"  >> $INSTLOG
LIBMODULE="/lib/modules/$KERNVER/kernel/drivers"
if [ ! -d "$LIBMODULE" ]; then
	echo " @ERR install directry($LIBMODULE) not found" >> $INSTLOG
	if [ -n "$2" ]; then
	echo "      Kernel $2 not installed." >> $INSTLOG
	fi
	exit 1
fi
echo "      Kernel version      : $KERNVER"   >> $INSTLOG
echo "      Host Type           : `uname -m`"   >> $INSTLOG
if [ "$1" = "install" ]; then
	echo " ---- Install  @Hitachi Fibre Channel Adapter Driver - `date`" >> $INSTLOG
#-------------
	Sptkver=(`ls $DRVDIR`)
	if [ "$HOSTTYPE" == "ia64" ]; then
		SMP=""
	else
		SMP="smp"
	fi
	KERNORG=`echo "$KERNVER" | cut -d . -f -3`.EL$SMP
	KERNUPD=`echo "$KERNVER" | cut -d . -f -3 | sed "s/\(.*\)-\(.*\)/\2/"`
	if [ $KERNUPD -lt 78 ]; then
		KERNORG=$KERNVER
	fi
	echo "        Target Kernel version               : $KERNVER" >> $INSTLOG
	echo "        Target Kernel version without patch : $KERNORG" >> $INSTLOG
	echo "                              update        : $KERNUPD" >> $INSTLOG
	found=0
	for dname in ${Sptkver[@]}; do
		if [ "$KERNORG" == "$dname" ]; then
			found=1
		fi
	done
	if [ "$found" == "1" ]; then
		if [ ! -f "$DRVDIR/$KERNORG/$DRVNAME" ]; then
			echo "       $DRVNAME is not found" >> $INSTLOG
			echo " @ERR install command failed."   >> $INSTLOG
			exit 1
		fi
                if [ ! -f "$DRVDIR/$KERNORG/$CNFNAME" ]; then
                        echo "       $CNFNAME is not found" >> $INSTLOG
                        echo " @ERR install command failed."   >> $INSTLOG
                        exit 1
                fi
                if [ ! -f "$DRVDIR/$KERNORG/$RLOGLINK" ]; then
                        echo "       $RLOGLINK is not found" >> $INSTLOG
                        echo " @ERR install command failed."   >> $INSTLOG
                        exit 1
                fi
	else
		echo "        hfcldd driver does not support kernel version $KERNORG" >> $INSTLOG
		echo " @ERR install command failed."   >> $INSTLOG
		exit 1
	fi
#------------
	if [ $KERNELMIN = "6" ]; then
		install -d -o root -g root /lib/modules/$KERNVER/updates
		if [ ! -f "$LIBMODULE/hraslog/hraslog.ko" ]; then
			echo "        hraslog.ko is not installed"   >> $INSTLOG
			install -o root -g root $DRVDIR/$KERNORG/$RLOGLINK /lib/modules/$KERNVER/updates
		else
			echo "        hraslog.ko is already installed"   >> $INSTLOG
			install -o root -g root $DRVDIR/$KERNORG/$RLOGLINK /lib/modules/$KERNVER/updates/$RLOGLINK.hraslog.rename
		fi
	fi
	echo "        install -d -o root -g root $LIBMODULE/scsi/" >> $INSTLOG
	install -d -o root -g root $LIBMODULE/scsi/
	echo "        install -o root -g root $DRVDIR/$KERNORG/$DRVNAME $LIBMODULE/scsi/"   >> $INSTLOG
	echo "        - backup current hfcldd files to *.backup"   >> $INSTLOG
	find /lib/modules/$KERNVER -name $DRVNAME -exec mv {} {}.backup \;
	find /lib/modules/$KERNVER -name $CNFNAME -exec mv {} {}.backup \;
	install -o root -g root $DRVDIR/$KERNORG/$DRVNAME $LIBMODULE/scsi/
	if [ ! -f "$LIBMODULE/scsi/$DRVNAME" ]; then
		echo " @ERR install command failed."   >> $INSTLOG
		exit 1
	fi
	echo "        install -o root -g root $DRVDIR/$KERNORG/$CNFNAME $LIBMODULE/scsi/"   >> $INSTLOG
	install -o root -g root $DRVDIR/$KERNORG/$CNFNAME $LIBMODULE/scsi/	
	if [ ! -f "/etc/hfcldd.conf" ]; then
		echo " --- Create /etc/hfcldd.conf "   >> $INSTLOG
		echo " " > /etc/hfcldd.conf   >> $INSTLOG
	fi
	if [ ! -f "$LIBMODULE/scsi/$CNFNAME" ]; then
		echo " @ERR install command failed (hfcldd_conf.o)."   >> $INSTLOG
		exit 1
	fi
	echo "      Modify /etc/$MODCONF; old file is /etc/$MODCONF.backup"   >> $INSTLOG
	cp /etc/$MODCONF /etc/$MODCONF.backup
	sed '/^#/s/scsi_hostadapter/hoge_hogehoge/g' /etc/$MODCONF > /etc/$MODCONF.temp
	mv /etc/$MODCONF.temp /etc/$MODCONF
	if [ `grep -c "scsi_hostadapter[0-9]*[[:space:]]\+hfcldd" /etc/$MODCONF` -eq 0 ]; then
		COUNT=`grep -c "scsi_hostadapter" /etc/$MODCONF`
		if [ $COUNT -eq 0 ]; then
			echo "alias scsi_hostadapter0 hfcldd_conf" >> /etc/$MODCONF
			echo "alias scsi_hostadapter1 hfcldd" >> /etc/$MODCONF
		elif [ $COUNT -eq 1 ]; then
			#########################################################################################################
			#
			# the following cases:
			#  1. alias scsi_hostadapter xxxxxx
			#
			#  2. alias scsi_hostadapterN xxxxxx
			#
			# after modified:
			#  1. alias scsi_hostadapter xxxxxx
			#     alias scsi_hostadapter1 hfcldd_conf
			#     alias scsi_hostadapter2 hfcldd
			#
			#  2. alias scsi_hostadapterN xxxxxx
			#     alias scsi_hostadapterN+1 hfcldd_conf
			#     alias scsi_hostadapterN+2 hfcldd
			#
			#########################################################################################################
			if [ `grep -c "scsi_hostadapter[ ]" /etc/$MODCONF` -eq 0 ]; then
				COUNT=`sed -n s/alias[[:space:]]*scsi_hostadapter//p /etc/$MODCONF|awk 'BIGIN{n=0};$1>n{n=$1};END{print n}'`
				LN=`sed -n /scsi_hostadapter0*$COUNT/{=} /etc/$MODCONF`
				COUNT=`expr $COUNT + 1`
				sed "$LN"a"alias scsi_hostadapter$COUNT hfcldd_conf" /etc/$MODCONF > /etc/$MODCONF.temp
				COUNT=`expr $COUNT + 1`
				LN=`expr $LN + 1`
				cp /etc/$MODCONF.temp /etc/$MODCONF
				sed "$LN"a"alias scsi_hostadapter$COUNT hfcldd" /etc/$MODCONF > /etc/$MODCONF.temp
				mv /etc/$MODCONF.temp /etc/$MODCONF
			else
				LN=`sed -n /scsi_hostadapter/{=} /etc/$MODCONF`
				sed "$LN"a"alias scsi_hostadapter$COUNT hfcldd_conf" /etc/$MODCONF > /etc/$MODCONF.temp
				cp /etc/$MODCONF.temp /etc/$MODCONF
				COUNT=`expr $COUNT + 1`
				LN=`expr $LN + 1`
				sed "$LN"a"alias scsi_hostadapter$COUNT hfcldd" /etc/$MODCONF > /etc/$MODCONF.temp
				mv /etc/$MODCONF.temp /etc/$MODCONF
			fi
		else
			#########################################################################################################
			#
			# the following cases:
			#  1. alias scsi_hostadapter xxxxxx
			#     alias scsi_hostadapterN xxxxxx
			#
			#  2. alias scsi_hostadapterM xxxxxx
			#     alias scsi_hostadapterN xxxxxx
			#
			# after modified:
			#  1. alias scsi_hostadapter xxxxxx
			#     alias scsi_hostadapterN+1 hfcldd_conf
			#     alias scsi_hostadapterN+2 hfcldd
			#
			#  2. alias scsi_hostadapterM xxxxxx
			#     alias scsi_hostadapterN+1 hfcldd_conf
			#     alias scsi_hostadapterN+2 hfcldd
			#
			#########################################################################################################
			cp /etc/$MODCONF /etc/$MODCONF.temp
			ADAP=`grep -c "scsi_hostadapter[ ]" /etc/$MODCONF`
			ADAP0=`grep -c "scsi_hostadapter0*0[ ]" /etc/$MODCONF`
			if [ $ADAP -gt 2 ] || [ $ADAP0 -gt 2 ]; then
				echo " @ERR double assigned scsi_hostadapter number" >> $INSTLOG
				sed '/^#/s/hoge_hogehoge/scsi_hostadapter/g' /etc/$MODCONF > /etc/$MODCONF.temp
				mv /etc/$MODCONF.temp /etc/$MODCONF
				exit 1
			fi
			if [ $ADAP -eq 1 ]; then
				# intentional character
				sed s/"scsi_hostadapter "/"scsi_xxxxxxxxxxx "/ /etc/$MODCONF > /etc/$MODCONF.temp
			fi
			COUNT=`sed -n s/alias[[:space:]]*scsi_hostadapter//p /etc/$MODCONF.temp|awk 'BIGIN{n=0};$1>n{n=$1};END{print n}'`
			LN=`sed -n /scsi_hostadapter0*$COUNT/{=} /etc/$MODCONF.temp`
			COUNT=`expr $COUNT + 1`
			sed "$LN"a"alias scsi_hostadapter$COUNT hfcldd_conf" /etc/$MODCONF > /etc/$MODCONF.temp
			COUNT=`expr $COUNT + 1`
			LN=`expr $LN + 1`
			cp /etc/$MODCONF.temp /etc/$MODCONF
			sed "$LN"a"alias scsi_hostadapter$COUNT hfcldd" /etc/$MODCONF > /etc/$MODCONF.temp
			mv /etc/$MODCONF.temp /etc/$MODCONF
		fi
	else
		if [ `grep -c "scsi_hostadapter[0-9]*[[:space:]]\+hfcldd_mp" /etc/$MODCONF` -eq 1 ]; then
			# delete hfcldd_mp line
			sed "s/\(scsi_hostadapter.*\) hfcldd_conf/\1 temp_conf/" /etc/$MODCONF > /etc/$MODCONF.temp
			sed "s/\(scsi_hostadapter.*\) hfcldd_mp/\1 temp_mp/" /etc/$MODCONF.temp > /etc/$MODCONF.temp2
			sed '/alias scsi_hostadapter.* hfcldd/d' /etc/$MODCONF.temp2 > /etc/$MODCONF.temp
			sed "s/\(scsi_hostadapter.*\) temp_conf/\1 hfcldd_conf/" /etc/$MODCONF.temp > /etc/$MODCONF.temp2
			sed "s/\(scsi_hostadapter.*\) temp_mp/\1 hfcldd/" /etc/$MODCONF.temp2 > /etc/$MODCONF
			rm /etc/$MODCONF.temp*
		fi
		if [ `grep -c "scsi_hostadapter[0-9]*[[:space:]]\+hfcldd_conf" /etc/$MODCONF` -eq 0 ]; then
			cp /etc/$MODCONF /etc/$MODCONF.temp
			ADAP=`grep -c "scsi_hostadapter[ ]" /etc/$MODCONF`						# number of "alias scsi_hostadapter"
			HADAP=`grep -c "scsi_hostadapter[[:space:]]\+hfcldd" /etc/$MODCONF`		# number of "alias scsi_hostadapter hfcldd"
			ADAP0=`grep -c "scsi_hostadapter0*0[ ]" /etc/$MODCONF`					# number of "alias scsi_hostadapter0 or 00"
			if [ $ADAP -gt 2 ] || [ $ADAP0 -gt 2 ]; then
				echo " @ERR double assigned scsi_hostadapter number" >> $INSTLOG
				sed '/^#/s/hoge_hogehoge/scsi_hostadapter/g' /etc/$MODCONF > /etc/$MODCONF.temp
				mv /etc/$MODCONF.temp /etc/$MODCONF
				exit 1
			fi
			if [ $ADAP -eq 1 ]; then
				if [ $HADAP -eq 0 ]; then
					# intentional character
					sed s/"scsi_hostadapter "/"scsi_xxxxxxxxxxx "/ /etc/$MODCONF > /etc/$MODCONF.temp
				else
					sed s/"scsi_hostadapter "/"scsi_hostadapter0 "/ /etc/$MODCONF > /etc/$MODCONF.temp
					cp /etc/$MODCONF.temp /etc/$MODCONF
				fi
			fi
			LO=`sed -n s/alias[[:space:]]*scsi_hostadapter//p /etc/$MODCONF.temp|awk 'BIGIN{n=0};$1>n{n=$1};END{print n}'`
			TMP=`grep -n "scsi_hostadapter[0-9]*[[:space:]]\+hfcldd" /etc/$MODCONF.temp`
			LN=`echo $TMP | awk '{split($0,a,":"); print a[1]}'`
			CONFNO=`echo $TMP | awk '{split($0,a,":"); print a[2]}'| awk '{print $2}'|sed s/scsi_hostadapter//`
			LO=`expr $LO + 1`
			ENDNO=`expr $CONFNO`
			while [ $LO -gt $ENDNO ]; do
				LI=`expr $LO - 1`
				if [ $ADAP -eq 1 ] && [ $HADAP -eq 1 ] && [ $ADAP0 -eq 1 ]; then
					#########################################################################################################
					#
					# the following cases:
					#   alias scsi_hostadapter hfcldd
					#   alias scsi_hostadapter0 xxxxxx  (alias scsi_hostadapter00 xxxxxx)
					#   alias scsi_hostadapterN xxxxxx
					#
					# after modified:
					#   alias scsi_hostadapter0 hfcldd_conf
					#   alias scsi_hostadapter1 hfcldd
					#	alias scsi_hostadapter2 xxxxxx  (alias scsi_hostadapter02 xxxxxx)
					#   alias scsi_hostadapterN+2 xxxxxx
					#
					#########################################################################################################
					LP=`expr $LO + 1`
					if [ $LI -eq 0 ]; then
						sed "s/\(scsi_hostadapter0*\)$LI[[:space:]]\+hfcldd/\1$LO hfcldd/" /etc/$MODCONF > /etc/$MODCONF.temp2
						if [ `grep -c "scsi_hostadapter00[ ]" /etc/$MODCONF` -eq 0 ]; then
							sed s/"scsi_hostadapter0 "/"scsi_hostadapter$LP "/ /etc/$MODCONF.temp2 > /etc/$MODCONF.temp
						else
							sed s/"scsi_hostadapter00 "/"scsi_hostadapter0$LP "/ /etc/$MODCONF.temp2 > /etc/$MODCONF.temp
						fi
						rm /etc/$MODCONF.temp2
					elif [ $LI -gt 9 ]; then
						# scsi_hostadapterXX: XX=10-
						sed "s/\(scsi_hostadapter\)$LI[[:space:]]/\1$LP /" /etc/$MODCONF > /etc/$MODCONF.temp
					else
						if [ `grep -c "scsi_hostadapter0$LI[ ]" /etc/$MODCONF` -eq 0 ]; then
							# scsi_hostadapterXX: XX=1digit, X=0-9
							sed "s/\(scsi_hostadapter\)$LI[[:space:]]/\1$LP /" /etc/$MODCONF > /etc/$MODCONF.temp
						elif [ $LI -eq 8 ] || [ $LI -eq 9 ]; then
							# scsi_hostadapterXX: XX=2digit, XX=08 or 09
							sed "s/scsi_hostadapter0$LI[[:space:]]/scsi_hostadapter$LP /" /etc/$MODCONF > /etc/$MODCONF.temp
						else
							# scsi_hostadapterXX: XX=2digit, XX=00-07
							sed "s/\(scsi_hostadapter0\)$LI[[:space:]]/\1$LP /" /etc/$MODCONF > /etc/$MODCONF.temp
						fi
					fi
				else
					#########################################################################################################
					#
					# the following cases:
					#  1. alias scsi_hostadapter hfcldd
					#     alias scsi_hostadapterN xxxxxx
					#
					#  2. alias scsi_hostadapter xxxxxx
					#     alias scsi_hostadapterN hfcldd
					#
					#  3. alias scsi_hostadapterN hfcldd
					#
					# after modified:
					#  1. alias scsi_hostadapter0 hfcldd_conf
					#     alias scsi_hostadapter1 hfcldd
					# 	  alias scsi_hostadapterN+1 xxxxxx
					#  
					#  2. alias scsi_hostadapter xxxxxx
					#     alias scsi_hostadapterN hfcldd_conf
					#     alias scsi_hostadapterN+1 hfcldd
					#
					#  3. alias scsi_hostadapterN hfcldd_conf
					#     alias scsi_hostadapterN+1 hfcldd
					#
					#########################################################################################################
					if [ $LI -gt 9 ]; then
						sed "s/\(scsi_hostadapter\)$LI[[:space:]]/\1$LO /" /etc/$MODCONF > /etc/$MODCONF.temp
					else
						if [ `grep -c "scsi_hostadapter0$LI[ ]" /etc/$MODCONF` -eq 0 ]; then
							# scsi_hostadapterXX: XX=1digit, X=0-9
							sed "s/\(scsi_hostadapter\)$LI[[:space:]]/\1$LO /" /etc/$MODCONF > /etc/$MODCONF.temp
						elif [ $LI -eq 9 ]; then
							# scsi_hostadapterXX: XX=2digit, XX=09
							sed "s/scsi_hostadapter0$LI[[:space:]]/scsi_hostadapter$LO /" /etc/$MODCONF > /etc/$MODCONF.temp
						else
							# scsi_hostadapterXX: XX=2digit, XX=00-08
							sed "s/\(scsi_hostadapter0\)$LI[[:space:]]/\1$LO /" /etc/$MODCONF > /etc/$MODCONF.temp
						fi
					fi
				fi
				cp /etc/$MODCONF.temp /etc/$MODCONF
				LO=`expr $LO - 1`
			done
			if [ $LN -gt 1 ]; then
				LN=`expr $LN - 1`
				sed "$LN"a"alias scsi_hostadapter$CONFNO hfcldd_conf" /etc/$MODCONF > /etc/$MODCONF.temp
			else
				sed "$LN"i"alias scsi_hostadapter$CONFNO hfcldd_conf" /etc/$MODCONF > /etc/$MODCONF.temp
			fi
   	    	mv /etc/$MODCONF.temp /etc/$MODCONF
		fi
	fi
	sed '/^#/s/hoge_hogehoge/scsi_hostadapter/g' /etc/$MODCONF > /etc/$MODCONF.temp
	mv /etc/$MODCONF.temp /etc/$MODCONF
	cp /$BOOTDIR/initrd-$KERNVER.img /$BOOTDIR/initrd-$KERNVER.img.backup
	echo "      Update /lib/modules/$KERNVER/modules.dep"   >> $INSTLOG
	echo "        /sbin/depmod -a $KERNVER"   >> $INSTLOG
	/sbin/depmod -a $KERNVER
	echo "      Update boot image file /$BOOTDIR/initrd-$KERNVER.img"         >> $INSTLOG
	echo "      old boot image file --> /$BOOTDIR/initrd-$KERNVER.img.backup" >> $INSTLOG
	echo "        /sbin/mkinitrd -f /$BOOTDIR/initrd-$KERNVER.img $KERNVER"   >> $INSTLOG
	/sbin/mkinitrd -f /$BOOTDIR/initrd-$KERNVER.img $KERNVER
	if [ $? -ne 0 ]; then
		echo " @ERR mkinitrd command fail"   >> $INSTLOG
		cp /$BOOTDIR/initrd-$KERNVER.img.backup /$BOOTDIR/initrd-$KERNVER.img
		rm -f /$BOOTDIR/initrd-$KERNVER.img.backup
		exit 1
	fi 
#	if [ "$KERNVER" = `uname -r` ]; then
#		echo "      check loaded modules"   >> $INSTLOG
#		COUNT=`/sbin/lsmod | grep hfcldd | wc -l`
#		NUM=`expr $COUNT`
#		if [ $NUM -eq 0 ]; then
#			if [ $KERNELMIN = "4" ]; then
#				echo "        /sbin/modprobe $CNFMODU"   >> $INSTLOG
#				/sbin/modprobe $CNFMODU  >> $INSTLOG
#				if [ $? -eq 1 ]; then
#					echo " @ERR modprobe command fail (hfcldd_conf.o)"   >> $INSTLOG
#					exit 1
#				fi
#				echo "        /sbin/modprobe $DRVMODU"   >> $INSTLOG
#				/sbin/modprobe $DRVMODU >> $INSTLOG
#				if [ $? -eq 1 ]; then
#					echo " @ERR modprobe command fail"   >> $INSTLOG
#					exit 1
#				fi
#			fi
#		else
			echo " ############################################################################"   >> $INSTLOG
#			echo "      A module named hfcldd* already exists"                                     >> $INSTLOG
			echo "      Skip insmod. Need reload a module again or reboot to update"               >> $INSTLOG
			echo " ############################################################################"   >> $INSTLOG
#		fi
#	fi
	install -d -o root -g root $TOOLSDIR
	cp hfcinst.sh /opt/hitachi/drivers/hba 
	cp COPYING /opt/hitachi/drivers/hba
#       cp README_hfcldd.txt /opt/hitachi/drivers/hba
        cp version_hfcldd.txt /opt/hitachi/drivers/hba
	echo " ---- Install Success "   >> $INSTLOG
else
	echo " ---- Uninstall  @Hitachi Fibre Channel Adapter Driver - `date`"   > $INSTLOG
	if [ $KERNELMIN = "6" ]; then
		if [ ! -f "$LIBMODULE/hraslog/hraslog.ko" ]; then
			rm -f /lib/modules/$KERNVER/updates/$RLOGLINK
		else
			rm -f /lib/modules/$KERNVER/updates/$RLOGLINK.hraslog.rename
		fi
	fi
	echo "        rm -f $LIBMODULE/scsi/$DRVNAME"   >> $INSTLOG
	rm -f $LIBMODULE/scsi/$DRVNAME
	echo "      Modify /etc/$MODCONF"   >> $INSTLOG
	sed '/^#/s/scsi_hostadapter/hoge_hogehoge/g' /etc/$MODCONF > /etc/$MODCONF.temp
	sed '/alias scsi_hostadapter.* hfcldd/d' /etc/$MODCONF.temp > /etc/$MODCONF
	rm /etc/$MODCONF.temp
	rm -f $LIBMODULE/scsi/$CNFNAME
	cp /etc/$MODCONF /etc/$MODCONF.temp
	sed '/alias scsi_hostadapter.* hfcldd_conf/d' /etc/$MODCONF.temp > /etc/$MODCONF
	sed '/^#/s/hoge_hogehoge/scsi_hostadapter/g' /etc/$MODCONF > /etc/$MODCONF.temp
	mv /etc/$MODCONF.temp /etc/$MODCONF
	echo "      Update /lib/modules/$KERNVER/modules.dep"   >> $INSTLOG
	echo "        /sbin/depmod -a $KERNVER"   >> $INSTLOG
	/sbin/depmod -a $KERNVER
	echo "        /sbin/mkinitrd -f /$BOOTDIR/initrd-$KERNVER.img $KERNVER"   >> $INSTLOG
	cp /$BOOTDIR/initrd-$KERNVER.img /$BOOTDIR/initrd-$KERNVER.img.backup
	/sbin/mkinitrd -f /$BOOTDIR/initrd-$KERNVER.img $KERNVER
	if [ $? -ne 0 ]; then
		echo " @ERR mkinitrd command fail"   >> $INSTLOG
		cp /$BOOTDIR/initrd-$KERNVER.img.backup /$BOOTDIR/initrd-$KERNVER.img
		exit 1
	fi 
	if [ $KERNELMIN = "6" ]; then
		echo " @Warning skip rmmod command "   >> $INSTLOG
	else
		if [ "$KERNVER" = `uname -r` ]; then
			if [ `/sbin/lsmod | grep hfcldd | wc -l` -ne 0 ]; then
				echo "        /sbin/rmmod $DRVMODU"   >> $INSTLOG
				/sbin/rmmod $DRVMODU >> $INSTLOG
				if [ $? -eq 1 ]; then
					echo " @ERR rmmod command fail (hfcldd) "   >> $INSTLOG
					exit 1
				fi
			fi
			if [ `/sbin/lsmod | grep hfcldd_mp | wc -l` -eq 1 ]; then
				echo "        /sbin/rmmod $MPMODU"   >> $INSTLOG
				/sbin/rmmod $MPMODU >> $INSTLOG
				if [ $? -eq 1 ]; then
					echo " @ERR rmmod command fail (hfcldd_mp) "   >> $INSTLOG
					exit 1
				fi
			fi
			if [ `/sbin/lsmod | grep hfcldd_conf | wc -l` -eq 1 ]; then
				echo "        /sbin/rmmod $CNFMODU"   >> $INSTLOG
				/sbin/rmmod $CNFMODU >> $INSTLOG
				if [ $? -eq 1 ]; then
					echo " @ERR rmmod command fail (hfcldd_conf) "   >> $INSTLOG
					exit 1
				fi
			fi
		fi 
	fi
	echo "        remove hfcinst.sh from /opt/hitachi/drivers/hba"   >> $INSTLOG
	rm -f $TOOLSDIR/hfcinst.sh
	rm -f $TOOLSDIR/COPYING
#	rm -f $TOOLSDIR/README_hfcldd.txt
	rm -f $TOOLSDIR/version_hfcldd.txt
    echo " ---- Uninstall Success "   >> $INSTLOG
fi
